function [Tour,Cost,time] = tsp_sat(G, maxCost);
   [t0, u0, s0] = cputime ();
   n = size(G,1);
   [Tour,Cost] = hamilton_sat(G,[1],0,2:n,maxCost);
   [t1, u1, s1] = cputime ();
   time = u1-u0;
end

function [Tour,Cost] = hamilton_sat(G, Visited, CostSoFar, ToVisit, maxCost)
   if length(ToVisit) == 0
      Tour = Visited;
      Cost = CostSoFar + G(Visited(end), Visited(1));
      if Cost > maxCost
         Cost = Inf;
      end
   else
      Cost = Inf;
      Tour = [];
      i = 0;
      while i < length(ToVisit) && Cost > maxCost
         i = i + 1;
         last = Visited(end);
         next = ToVisit(i);
         NewVisited = [Visited,next];
         NewCostSoFar = CostSoFar + G(last, next);
         NewToVisit = [ToVisit(1:i-1), ToVisit(i+1:end)];
         [T,C] = hamilton_sat(G, NewVisited, NewCostSoFar, NewToVisit, maxCost);
         if C <= maxCost
            Tour = T; 
            Cost = C;
         end
      end
   end
end