function [Tour,Cost,time] = tsp_sat_h(G, maxCost);
   [t0, u0, s0] = cputime ();
   n = size(G,1);
   [Tour,Cost] = hamilton_sat_h(G,[1],0,2:n,maxCost);
   [t1, u1, s1] = cputime ();
   time = u1-u0;
end

function [Tour,Cost] = hamilton_sat_h(G, Visited, CostSoFar, ToVisit, maxCost)
   if length(ToVisit) == 0
      Tour = Visited;
      Cost = CostSoFar + G(Visited(end), Visited(1));
      if Cost > maxCost
         Cost = Inf;
      end
   else
      Cost = Inf;
      Tour = [];
      i = 0;
      while i < length(ToVisit) && Cost > maxCost
         i = i + 1;
         last = Visited(end);
         next = nextBest(i,last,ToVisit,G);
         NewCostSoFar = CostSoFar + G(last, next);
         NewVisited = [Visited,next];
         NewToVisit = remove(next, ToVisit);
         [T,C] = hamilton_sat_h(G, NewVisited, NewCostSoFar, NewToVisit, maxCost);
         if C <= maxCost
            Tour = T; 
            Cost = C;
         end
      end
   end
end

function NewList = remove(j, List)
   NewList = [];
   for v = List
      if v != j
         NewList = [NewList,v];
      end
   end
end

function j = nextBest(i,last,ToVisit,G)
   Next = sortNext(G, last, ToVisit); % 1:length(ToVisit)
   j = Next(i);
end

function Next = sortNext(G,last,Nodes)
   n = length(Nodes);
   Next = Nodes;
   for k = n:-1:2
      for i = 1:k-1
         if G(last,Next(i)) > G(last, Next(i+1))
            x = Next(i);
            Next(i) = Next(i+1);
            Next(i+1) = x;
         end
      end
   end
end